/*
 * (C) Copyright 2000-2002
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * (C) Copyright 2001 Sysgo Real-Time Solutions, GmbH <www.elinos.com>
 * Andreas Heppel <aheppel@sysgo.de>

 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <command.h>
#include <environment.h>
#include <linux/stddef.h>
#include <asm/byteorder.h>
#include <malloc.h>
#include "mvGpp.h"

#ifdef CONFIG_SHOW_BOOT_PROGRESS
# include <status_led.h>
# define SHOW_BOOT_PROGRESS(arg)	show_boot_progress(arg)
#else
# define SHOW_BOOT_PROGRESS(arg)
#endif

#ifdef CONFIG_AMIGAONEG3SE
	extern void enable_nvram(void);
	extern void disable_nvram(void);
#endif

#undef DEBUG_ENV
#ifdef DEBUG_ENV
#define DEBUGF(fmt,args...) printf(fmt ,##args)
#else
#define DEBUGF(fmt,args...)
#endif

extern env_t *env_ptr;

extern void env_relocate_spec (void);
extern uchar env_get_char_spec(int);

static uchar env_get_char_init (int index);
uchar (*env_get_char)(int) = env_get_char_init;

char g_szEnvEthOld[sizeof(ETHADDR)];

/************************************************************************
 * Default settings to be used when no valid environment is found
 */
#define XMK_STR(x)	#x
#define MK_STR(x)	XMK_STR(x)

uchar default_environment[] = {
#if defined CONFIG_BUFFALO_PLATFORM
	"ethaddr="	ETHADDR				"\0"
#else
 #ifdef	CONFIG_ETHADDR
	"ethaddr="	MK_STR(CONFIG_ETHADDR)		"\0"
 #endif
#endif
#ifdef	CONFIG_BOOTARGS
	"bootargs="	CONFIG_BOOTARGS			"\0"
#endif
#ifdef	CONFIG_BOOTCOMMAND
	"bootcmd="	CONFIG_BOOTCOMMAND		"\0"
#endif
#ifdef	CONFIG_RAMBOOTCOMMAND
	"ramboot="	CONFIG_RAMBOOTCOMMAND		"\0"
#endif
#ifdef	CONFIG_NFSBOOTCOMMAND
	"nfsboot="	CONFIG_NFSBOOTCOMMAND		"\0"
#endif
#if defined(CONFIG_BOOTDELAY) && (CONFIG_BOOTDELAY >= 0)
	"bootdelay="	MK_STR(CONFIG_BOOTDELAY)	"\0"
#endif
#if defined(CONFIG_BAUDRATE) && (CONFIG_BAUDRATE >= 0)
	"baudrate="	MK_STR(CONFIG_BAUDRATE)		"\0"
#endif
#ifdef	CONFIG_LOADS_ECHO
	"loads_echo="	MK_STR(CONFIG_LOADS_ECHO)	"\0"
#endif
#ifdef	CONFIG_ETH1ADDR
	"eth1addr="	MK_STR(CONFIG_ETH1ADDR)		"\0"
#endif
#ifdef	CONFIG_ETH2ADDR
	"eth2addr="	MK_STR(CONFIG_ETH2ADDR)		"\0"
#endif
#ifdef	CONFIG_IPADDR
	"ipaddr="	MK_STR(CONFIG_IPADDR)		"\0"
#endif
#ifdef	CONFIG_SERVERIP
	"serverip="	MK_STR(CONFIG_SERVERIP)		"\0"
#endif
#ifdef	CFG_AUTOLOAD
	"autoload="	CFG_AUTOLOAD			"\0"
#endif
#ifdef	CONFIG_PREBOOT
	"preboot="	CONFIG_PREBOOT			"\0"
#endif
#ifdef	CONFIG_ROOTPATH
	"rootpath="	MK_STR(CONFIG_ROOTPATH)		"\0"
#endif
#ifdef	CONFIG_GATEWAYIP
	"gatewayip="	MK_STR(CONFIG_GATEWAYIP)	"\0"
#endif
#ifdef	CONFIG_NETMASK
	"netmask="	MK_STR(CONFIG_NETMASK)		"\0"
#endif
#ifdef	CONFIG_HOSTNAME
	"hostname="	MK_STR(CONFIG_HOSTNAME)		"\0"
#endif
#ifdef	CONFIG_BOOTFILE
	"bootfile="	MK_STR(CONFIG_BOOTFILE)		"\0"
#endif
#ifdef	CONFIG_LOADADDR
	"loadaddr="	MK_STR(CONFIG_LOADADDR)		"\0"
#endif
#ifdef  CONFIG_CLOCKS_IN_MHZ
	"clocks_in_mhz=1\0"
#endif
#if defined(CONFIG_PCI_BOOTDELAY) && (CONFIG_PCI_BOOTDELAY > 0)
	"pcidelay="	MK_STR(CONFIG_PCI_BOOTDELAY)	"\0"
#endif
#ifdef  CONFIG_EXTRA_ENV_SETTINGS
	CONFIG_EXTRA_ENV_SETTINGS
#endif
	"\0"
};

#if defined(CFG_ENV_IS_IN_NAND)		/* Environment is in Nand Flash */
int default_environment_size = sizeof(default_environment);
#endif


void env_crc_update (void)
{
	env_ptr->crc = crc32(0, env_ptr->data, ENV_SIZE);
}

static uchar env_get_char_init (int index)
{
	DECLARE_GLOBAL_DATA_PTR;
	uchar c;

	/* if crc was bad, use the default environment */
	if (gd->env_valid)
	{
		c = env_get_char_spec(index);
	} else {
		c = default_environment[index];
	}

	return (c);
}

#ifdef CONFIG_AMIGAONEG3SE
uchar env_get_char_memory (int index)
{
	DECLARE_GLOBAL_DATA_PTR;
	uchar retval;
	enable_nvram();
	if (gd->env_valid) {
		retval = ( *((uchar *)(gd->env_addr + index)) );
	} else {
		retval = ( default_environment[index] );
	}
	disable_nvram();
	return retval;
}
#else
uchar env_get_char_memory (int index)
{
	DECLARE_GLOBAL_DATA_PTR;

	if (gd->env_valid) {
		return ( *((uchar *)(gd->env_addr + index)) );
	} else {
		return ( default_environment[index] );
	}
}
#endif

uchar *env_get_addr (int index)
{
	DECLARE_GLOBAL_DATA_PTR;

	if (gd->env_valid) {
		return ( ((uchar *)(gd->env_addr + index)) );
	} else {
		return (&default_environment[index]);
	}
}

void env_relocate (void)
{
	DECLARE_GLOBAL_DATA_PTR;

	DEBUGF ("%s[%d] offset = 0x%lx\n", __FUNCTION__,__LINE__,
		gd->reloc_off);

#ifdef CONFIG_AMIGAONEG3SE
	enable_nvram();
#endif

#ifdef ENV_IS_EMBEDDED
	/*
	 * The environment buffer is embedded with the text segment,
	 * just relocate the environment pointer
	 */
	env_ptr = (env_t *)((ulong)env_ptr + gd->reloc_off);
	DEBUGF ("%s[%d] embedded ENV at %p\n", __FUNCTION__,__LINE__,env_ptr);
#else
	/*
	 * We must allocate a buffer for the environment
	 */
	env_ptr = (env_t *)malloc (CFG_ENV_SIZE);
	DEBUGF ("%s[%d] malloced ENV at %p\n", __FUNCTION__,__LINE__,env_ptr);
#endif

	/*
	 * After relocation to RAM, we can always use the "memory" functions
	 */
	env_get_char = env_get_char_memory;

	if (gd->env_valid == 0) {
#if defined(CONFIG_GTH)	|| defined(CFG_ENV_IS_NOWHERE)	/* Environment not changable */
		puts ("Using default environment\n\n");
#else
		puts ("*** Warning - bad CRC, using default environment\n\n");
		SHOW_BOOT_PROGRESS (-1);
#endif

		if (sizeof(default_environment) > ENV_SIZE)
		{
			puts ("*** Error - default environment is too large\n\n");
			return;
		}

		memset (env_ptr, 0, sizeof(env_t));
		memcpy (env_ptr->data,
			default_environment,
			sizeof(default_environment));
#ifdef CFG_REDUNDAND_ENVIRONMENT
		env_ptr->flags = 0xFF;
#endif
		env_crc_update ();
		gd->env_valid = 1;
#ifdef CONFIG_BUFFALO_PLATFORM
		gd->nvram_default = 1;
#endif
	}
	else {
		env_relocate_spec ();
	}
	gd->env_addr = (ulong)&(env_ptr->data);

#ifdef CONFIG_AMIGAONEG3SE
	disable_nvram();
#endif
}

#ifdef CONFIG_AUTO_COMPLETE
int env_complete(char *var, int maxv, char *cmdv[], int bufsz, char *buf)
{
	int i, nxt, len, vallen, found;
	const char *lval, *rval;

	found = 0;
	cmdv[0] = NULL;

	len = strlen(var);
	/* now iterate over the variables and select those that match */
	for (i=0; env_get_char(i) != '\0'; i=nxt+1) {

		for (nxt=i; env_get_char(nxt) != '\0'; ++nxt)
			;

		lval = env_get_addr(i);
		rval = strchr(lval, '=');
		if (rval != NULL) {
			vallen = rval - lval;
			rval++;
		} else
			vallen = strlen(lval);

		if (len > 0 && (vallen < len || memcmp(lval, var, len) != 0))
			continue;

		if (found >= maxv - 2 || bufsz < vallen + 1) {
			cmdv[found++] = "...";
			break;
		}
		cmdv[found++] = buf;
		memcpy(buf, lval, vallen); buf += vallen; bufsz -= vallen;
		*buf++ = '\0'; bufsz--;
	}

	cmdv[found] = NULL;
	return found;
}
#endif

extern MV_U32 boardIdToIndex[];
extern unsigned int mvBoardIdGet(void);
extern MV_BOARD_INFO mv88F5182InfoTbl[];
#define BOARD_INFO boardInfoTbl[boardIdToIndex[boardId]]
//#define PRODUCT_ID PID_LS_CHL
int init_board_info(MV_BOOL force_init)
{
	MV_BOARD_INFO *boardInfoTbl = mv88F5182InfoTbl;
	unsigned int boardId = mvBoardIdGet();
	char *envp = env_get_addr(CFG_ENV_SIZE - 4 - CFG_ENV_B_SIZE);
	int i;

	if (!force_init
//	    && (*((u32 *)(envp + BUF_NV_ADDR_PID)) == __cpu_to_be32(PRODUCT_ID))
	    && *((u32 *)(envp + BUF_NV_ADDR_VERSION)) == __cpu_to_be32(CFG_ENV_B_VERSION))
		return 0;

	*((u32 *)(envp + BUF_NV_ADDR_VERSION)) = __cpu_to_be32(CFG_ENV_B_VERSION);
//	*((u32 *)(envp + BUF_NV_ADDR_PID)) = __cpu_to_be32(PRODUCT_ID);

	*((u32 *)(envp + BUF_NV_ADDR_INTS_GPP_MASK)) = __cpu_to_be32(BOARD_INFO.intsGppMask);
	*(envp + BUF_NV_ADDR_NUM_BOARD_MPP) = DB_BUF_BOARD_DEFAULT_MPP_CONFIG_NUM;
	*(envp + BUF_NV_ADDR_NUM_BOARD_DEVICE_IF) = DB_BUF_BOARD_DEFAULT_DEVICE_CONFIG_NUM;
	*(envp + BUF_NV_ADDR_NUM_BOARD_PCI_IF) = DB_BUF_BOARD_DEFAULT_PCI_IF_NUM;
	*(envp + BUF_NV_ADDR_NUM_BOARD_TWSI_DEV) = DB_BUF_BOARD_DEFAULT_TWSI_DEF_NUM;
	*(envp + BUF_NV_ADDR_NUM_BOARD_MAC_INFO) = DB_BUF_BOARD_DEFAULT_MAC_INFO_NUM;
	*(envp + BUF_NV_ADDR_NUM_BOARD_GPP_INFO) = DB_BUF_BOARD_DEFAULT_GPP_INFO_NUM;
	*(envp + BUF_NV_ADDR_ACTIVE_LEDS_NUMBER) = DB_BUF_BOARD_DEFAULT_DEBUG_LED_NUM;
	*(envp + BUF_NV_ADDR_LEDS_POLARITY) = 0;

	//	*((u32 *)(envp + BUF_NV_ADDR_GPP_OUT_EN_VAL)) = __cpu_to_be32(MV_REG_READ(GPP_DATA_OUT_EN_REG(0)));
	*((u32 *)(envp + BUF_NV_ADDR_GPP_OUT_VAL)) = __cpu_to_be32(MV_REG_READ(GPP_DATA_OUT_REG(0)));
	//	*((u32 *)(envp + BUF_NV_ADDR_GPP_POLARITY_VAL)) = __cpu_to_be32(MV_REG_READ(GPP_DATA_IN_POL_REG(0)));

	*((u32 *)(envp + BUF_NV_ADDR_GPP_OUT_EN_VAL)) = __cpu_to_be32(BUFFALO_BOARD_DEFAULT_GPP_OE);
	*((u32 *)(envp + BUF_NV_ADDR_GPP_POLARITY_VAL)) = __cpu_to_be32(BUFFALO_BOARD_DEFAULT_GPP_POLVAL);

//	strcpy(envp + BUF_NV_ADDR_SERIES_NAME, BUFFALO_BOARD_DEFAULT_SERIES_NAME);
//	strcpy(envp + BUF_NV_ADDR_PRODUCT_NAME, BUFFALO_BOARD_DEFAULT_PRODUCT_NAME);
	// MV_BOARD_MPP_INFO
	*((u32 *)(envp + BUF_NV_ADDR_BOARD_MPP_INFO)) = __cpu_to_be32(BOARD_INFO.mppGroup[0]);
	*((u32 *)(envp + BUF_NV_ADDR_BOARD_MPP_INFO + 0x4)) = __cpu_to_be32(BOARD_INFO.mppGroup[1]);
	*((u32 *)(envp + BUF_NV_ADDR_BOARD_MPP_INFO + 0x8)) = __cpu_to_be32(BOARD_INFO.mppGroup[2]);
	*((u32 *)(envp + BUF_NV_ADDR_BOARD_MPP_INFO + 0xC)) = __cpu_to_be32(BOARD_INFO.mppGroup[3]);
	// MV_DEV_CS_INFO
	*(envp + BUF_NV_ADDR_DEV_CS_INFO_0 + 0x0) = 1;					// deviceCS
	*(envp + BUF_NV_ADDR_DEV_CS_INFO_0 + 0x1) = BOARD_DEV_NOR_FLASH;		// devType
	*(envp + BUF_NV_ADDR_DEV_CS_INFO_0 + 0x2) = 8;					// params
	*((u32 *)(envp + BUF_NV_ADDR_DEV_CS_INFO_0 + 0x4)) = __cpu_to_be32(0x8FCFFFFF);	// devWidth
	*(envp + BUF_NV_ADDR_DEV_CS_INFO_1 + 0x0) = 3;					// deviceCS
	*(envp + BUF_NV_ADDR_DEV_CS_INFO_1 + 0x1) = BOARD_DEV_NOR_FLASH;		// devType
	*(envp + BUF_NV_ADDR_DEV_CS_INFO_1 + 0x2) = 8;					// params
	*((u32 *)(envp + BUF_NV_ADDR_DEV_CS_INFO_1 + 0x4)) = __cpu_to_be32(0x8FCFFFFF);	// devWidth
	// MV_BOARD_TWSI_INFO
	*(envp + BUF_NV_ADDR_BOARD_TWSI_INFO + 0x0) = BOARD_DEV_RTC;			// devClass
	*(envp + BUF_NV_ADDR_BOARD_TWSI_INFO + 0x1) = BOARD_INFO.rtcTwsiAddr;
	*(envp + BUF_NV_ADDR_BOARD_TWSI_INFO + 0x2) = BOARD_INFO.rtcTwsiAddrType;
	// MV_MAC_INFO
	*(envp + BUF_NV_ADDR_BOARD_MAC_INFO + 0x0) = BOARD_MAC_SPEED_AUTO;		// boardMacSpeed
	*(envp + BUF_NV_ADDR_BOARD_MAC_INFO + 0x1) = 0x8;				// boardEthSmiAddr

	for (i=0; i<BUF_NV_SIZE_BIT_HDD_POWER; i++) {
		*(envp + BUF_NV_ADDR_BIT_HDD_POWER + i) = -1;
	}
	*(envp + BUF_NV_ADDR_BIT_HDD_POWER + 0x0) = PIN_NUM_HDD_POWER;
	*(envp + BUF_NV_ADDR_BIT_HDD_POWER + 0x1) = PIN_NUM_HDD_POWER1;

	for (i=0; i<BUF_NV_SIZE_BIT_USB_POWER; i++) {
		*(envp + BUF_NV_ADDR_BIT_USB_POWER + i) = -1;
	}
	*(envp + BUF_NV_ADDR_BIT_USB_POWER + 0x0) = PIN_NUM_USB_POWER;
	*(envp + BUF_NV_ADDR_BIT_USB_POWER + 0x1) = PIN_NUM_USB_POWER2;
	
	for (i=0; i<BUF_NV_SIZE_BIT_OUTPUT; i++) {
		*(envp + BUF_NV_ADDR_BIT_OUTPUT + i) = -1;
	}
	*(envp + BUF_NV_ADDR_BIT_FAN_LOW) = PIN_NUM_FAN_LOW;
	*(envp + BUF_NV_ADDR_BIT_FAN_HIGH) = PIN_NUM_FAN_HIGH;

	*(envp + BUF_NV_ADDR_BIT_LED_POWER) = PIN_NUM_LED_POWER;
	*(envp + BUF_NV_ADDR_BIT_LED_INFO) = PIN_NUM_LED_INFO;
	*(envp + BUF_NV_ADDR_BIT_LED_ALARM) = PIN_NUM_LED_ALARM;
	*(envp + BUF_NV_ADDR_BIT_LED_FUNC) = PIN_NUM_LED_FUNC;
	*(envp + BUF_NV_ADDR_BIT_LED_ETH) = PIN_NUM_LED_ETH;	// dummy

	*(envp + BUF_NV_ADDR_LED_GRP_POWER) = LED_POWER_GRP;
	*(envp + BUF_NV_ADDR_LED_GRP_INFO) = LED_INFO_GRP;
	*(envp + BUF_NV_ADDR_LED_GRP_ALARM) = LED_ALARM_GRP;
	*(envp + BUF_NV_ADDR_LED_GRP_FUNC) = PIN_NUM_LED_FUNC;
	*(envp + BUF_NV_ADDR_LED_GRP_ETH) = LED_ETH_GRP;

	for (i=0; i<BUF_NV_SIZE_BIT_INPUT; i++) {
		*(envp + BUF_NV_ADDR_BIT_INPUT + i) = -1;
	}
	*(envp + BUF_NV_ADDR_SW_INIT) = PIN_NUM_SW_INIT;
	*(envp + BUF_NV_ADDR_SW_POWER) = PIN_NUM_SW_POWER;
	*(envp + BUF_NV_ADDR_SW_RAID) = PIN_NUM_SW_RAID;
	*(envp + BUF_NV_ADDR_SW_AUTO_POWER) = PIN_NUM_SW_AUTO_POWER;
	*(envp + BUF_NV_ADDR_SW_FUNC) = PIN_NUM_SW_FUNC;
	*(envp + BUF_NV_ADDR_BIT_FAN_LOCK) = PIN_NUM_FAN_LOCK;

	*(envp + BUF_NV_ADDR_USE_SWITCH_SLIDE_POWER) = USE_SWITCH_SLIDE_POWER;

	return 1;
}
